<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.fotobunny.com
 * @since             1.0.0
 * @package           Avos
 *
 * @wordpress-plugin
 * Plugin Name:       Avos
 * Plugin URI:        https://www.camgirlstats.com
 * Description:       Embed Avos Webcams within Wordpress
 * Version:           1.0.0
 * Author:            Fotobunny
 * Author URI:        https://www.fotobunny.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       avos
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */

define( 'AVOS_VERSION', '1.0.0' );
add_action('init', 'avos_init');

//
// to embed avos webcams within the site page uncomment the 'init' 
// action above and comment out the 'template_redirect' action
//

function avos_init() {

	global $_AVOS;

    //
    // queue up our actions
    //

    add_shortcode('avos', 'avos_shortcode');
    add_action('wp_enqueue_scripts', 'avos_enqueue_scripts');
    add_filter('document_title_parts', 'avos_change_page_title' );
    add_filter('pre_handle_404', 'avos_pre_handle_404', 10, 2);
    add_filter('wpseo_sitemap_index', 'avos_sitemaps');

    //
    // bootstrap avos
    //

    if (!defined('TOP')) {

        $config = parse_ini_file(dirname($_SERVER['DOCUMENT_ROOT']) . '/avos/config.ini', true);

        if (!isset($config['templates']['theme'])) {
            define('TEMPLATES', dirname(__FILE__) . '/templates/');
        }

        define('TOP', dirname($_SERVER['DOCUMENT_ROOT']) . '/avos/');
        require_once(TOP . 'bin/bootstrap.php');
    }

    if (get_option('permalink_structure')) {
        $url = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
    } 
    else {
        parse_str(parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY), $params);
        $url = (isset($params['page_id']) ? $params['page_id'] : false);
    }

    //
    // if wp_template is defined then the webcams html is used by that template else
    // the usual posts template is used
    //

    if (getConfig('templates', 'wp_template', '')) {
        add_action('template_redirect', 'avos_template_redirect');
    }
    else {
        $html = avos_webcams();

        if (!empty($html)) {
            add_filter('the_posts', 'avos_virtual_page');
        }
    }
}

//
// if url matches avos [urls] display within the wordpress template defined in the
// avos config.ini 'wp_template' in the [templates] block
//
// e.g:
//
// [templates]
// wp_template = page.php
//

function avos_template_redirect() {

    global $_AVOS;

    $html = avos_webcams();

    if (!empty($html)) {
        include(get_stylesheet_directory() . '/' . getConfig('templates', 'wp_template'));
        die;
    }
}

//
// add any css/js files from the avos theme folders:
//
// assets/css/
// assets/js/
//

function avos_enqueue_scripts() {

    global $_AVOS;

    $files = glob(TEMPLATES . 'assets/js/*');

    foreach ($files as $f) {
        wp_enqueue_script('avos-' . str_replace('.', '-', basename($f)), plugin_dir_url(__FILE__) . 'templates/assets/js/' . basename($f), '', time());
    }

    $files = glob(TEMPLATES . 'assets/css/*');

    foreach ($files as $f) {
        wp_enqueue_style('avos-' . str_replace('.', '-', basename($f)), plugin_dir_url(__FILE__) . 'templates/assets/css/' . basename($f), '', time());
    }
}

//
// prevent http 404 if url matches avos [urls]
//

function avos_pre_handle_404(bool $preempt, WP_Query $wp_query) {

    global $_AVOS;

    $html = avos_webcams();

    if (empty($html)) {
        return $preempt;
    }

    return true;
}

//
// used from avos_init() to fake a wordpress post/page
//

function avos_virtual_page($posts) {

	global $_AVOS, $wp, $wp_query;

	$post = avos_virtual_page_object('webcams', avos_get_page_title(), $_AVOS['wordpress'][$_SERVER['REQUEST_URI']]);

	// set filter results
	$posts = array($post);

	// reset wp_query properties to simulate a found page
	$wp_query->is_page = true;
	$wp_query->is_singular = true;
	$wp_query->is_home = false;
	$wp_query->is_archive = false;
	$wp_query->is_category = false;
	unset($wp_query->query['error']);
	$wp_query->query_vars['error'] = '';
	$wp_query->is_404 = false;

	return ($posts);
}

//
// structure for an empty wordpress virtual post/page
//

function avos_virtual_page_object($slug, $title, $content) {

    $post                        = new stdClass;
    $post->ID                    = -1;
    $post->post_author           = 1;
    $post->post_date             = current_time( 'mysql' );
    $post->post_date_gmt         = current_time( 'mysql', 1 );
    $post->post_content          = $content;
    $post->post_title            = $title;
    $post->post_excerpt          = '';
    $post->post_status           = 'publish';
    $post->comment_status        = 'closed';
    $post->ping_status           = 'closed';
    $post->post_password         = '';
    $post->post_name             = $slug;
    $post->to_ping               = '';
    $post->pinged                = '';
    $post->modified              = $post->post_date;
    $post->modified_gmt          = $post->post_date_gmt;
    $post->post_content_filtered = '';
    $post->post_parent           = 0;
    $post->guid                  = get_home_url( 1, '/' . $slug );
    $post->menu_order            = 0;
    $post->post_type             = 'page';
    $post->post_mime_type        = '';
    $post->comment_count         = 0;

    return $post;
}

//
// return the page title if set by avos
// used by shortcode via the pre_get_document_title filter
// and can be used within templates
//

function avos_get_page_title() {

    global $_AVOS;

    if (isset($_AVOS['page']['meta']['title'])) {
        return $_AVOS['page']['meta']['title'];
    }

    return '';
}

//
// call avos_webcams() from shortcode [avos]
//

function avos_shortcode($atts = []) {

    return avos_webcams($atts);
}

//
// match current or provided url against [urls] in avos and return matching content
//

function avos_webcams($atts = []) {

    global $_AVOS;

    if (isset($atts['url']) && !empty($atts['url'])) {
        $_SERVER['REQUEST_URI'] = $atts['url'];
    }

    //
    // see if we have the result cached
    //

    if (isset($_AVOS['wordpress'][$_SERVER['REQUEST_URI']])) {
        return $_AVOS['wordpress'][$_SERVER['REQUEST_URI']];
    }

    //
    // parse requested url
    //

    $html = Avos\Scaffold\route('Avos',
        Avos\Scaffold\decodeUrlConfig(), true
    );

    //
    // set page title
    //

    if (!empty($html)) {
        if (empty($_AVOS['page']['meta']['title'])) {
            $_AVOS['page']['meta']['title'] = "Webcams";
        }

        add_filter('pre_get_document_title', 'avos_get_page_title');
    }

    //
    // cache the result
    //

    $_AVOS['wordpress'][$_SERVER['REQUEST_URI']] = $html;

    //
    // output any html if a route was found
    //

    return $html;
}

//
// add avos sitemap files to the wordpress sitemap index
//

function avos_sitemaps($items) {

    global $_AVOS;

    $web = getConfig('site', 'webDir');
    $url = getConfig('site', 'url');
    $files = glob($web . 'sitemap.*.xml*');

    foreach ($files as $f) {
        if (preg_match('!/(sitemap.*xml*)!', $f, $c)) {

            $time = filemtime($f);

            $items .= "
            <sitemap>
            <loc>$url$c[1]</loc>
            <lastmod>" . date('Y-m-d h:i P', $time) . "</lastmod>
            </sitemap>";
        }
    }

    return $items;
}

/*
 * possible skeleton code for a widget

function hstngr_register_widget() {
    register_widget( 'hstngr_widget' );
}

add_action( 'widgets_init', 'hstngr_register_widget' );

class hstngr_widget extends WP_Widget {
    function __construct() {
        parent::__construct(
            // widget ID
            'hstngr_widget',
            // widget name
            __('Hostinger Sample Widget', ' hstngr_widget_domain'),
            // widget description
            array( 'description' => __( 'Hostinger Widget Tutorial', 'hstngr_widget_domain' ), )
        );
    }

    public function widget( $args, $instance ) {
        $title = apply_filters( 'widget_title', $instance['title'] );
        echo $args['before_widget'];
        //if title is present
        if ( ! empty( $title ) )
            echo $args['before_title'] . $title . $args['after_title'];
            //output
            echo __( 'Greetings from Hostinger.com!', 'hstngr_widget_domain' );
            echo $args['after_widget'];
    }

    public function form( $instance ) {
        if ( isset( $instance[ 'title' ] ) )
            $title = $instance[ 'title' ];
        else
            $title = __( 'Default Title', 'hstngr_widget_domain' );
        ?>
            <p>
            <label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label>
            <input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
            </p>
        <?php
    }

    public function update( $new_instance, $old_instance ) {
        $instance = array();
        $instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
        return $instance;
    }
}
*/

/*
add_action('template_redirect', 'avos_capture_404', - PHP_INT_MAX );

function avos_capture_404() {

    global $_AVOS, $wp_query;

    #if (!is_404()) {
    #    return;
    #}

    if (!isset($_AVOS['wordpress']) || empty($_AVOS['wordpress'])) {
        return;
    }

    #$wp_query->set_404();
    #$html = avos_webcams();

    #if (!empty($html)) {

        if (empty($_AVOS['page']['meta']['title'])) {
            $_AVOS['page']['meta']['title'] = "Webcams";
        }

        add_filter('pre_get_document_title', 'avos_change_page_title');

        #get_page_template();
        get_header();
        echo $html;
        get_sidebar();
        get_footer();
        exit();
    #}

    //
    // else drop thru and let wordpress issue the real 404
    //
}

add_action('init', function() {
    global $wp_query;
    $html = avos_webcams();
    if (!empty($html)) {
        status_header(200);
        $wp_query->is_page = true;
        $wp_query->is_404  = false;
        get_header();
        echo $html;
        get_sidebar();
        get_footer();
        exit();
    }
});
 */

// the template_redirect action will overide the custom 404 plugin
#add_action('template_redirect', 'avos_capture_404');
#add_filter('404_template', 'avos_capture_404', 9999999);

#add_action('shutdown', 'avos_capture_404');

/* does nothing
add_filter("the_content", function($html) {
	if ( is_singular() && in_the_loop() && is_main_query() ) {
        return "FOOOOOOOOOOOOOOO".$html;
	}
}, 99);

add_filter('generate_rewrite_rules', function ($wp_rewrite){
    $wp_rewrite->rules = array_merge(
        ['webcams/(\d+)?$' => 'index.php?avos=1'],
        $wp_rewrite->rules
    );
});

add_filter('query_vars', function($query_vars){
    $query_vars[] = 'avos';
    return $query_vars;
});
 */
